@@ -3,7 +3,6 @@
 # Integration authorization tests
 #
 # pylint: disable=consider-using-enumerate
-# pylint: disable=too-many-arguments
 # pylint: disable=too-many-locals
 # pylint: disable=too-many-nested-blocks
 
@@ -17,6 +16,11 @@ from okera._thrift_api import TRecordServiceException
 from okera._thrift_api import TTypeId
 from okera.tests import pycerebro_test_common as common
 
+SKIP_LEVELS = ["smoke", "dev", "all"]
+
+EPOCH0_DATE = '1970-01-01'
+EPOCH0_TS = '1970-01-01 00:00:00.000'
+
 TEST_DB = 'auth_test_db'
 TEST_ROLE = 'auth_test_role'
 TEST_USER = 'auth_test_user'
@@ -36,7 +40,6 @@ else:
     random.seed(1)
 SKIP_SCANS = common.get_bool_env_var('AUTH_TEST_SKIP_SCANS', False)
 AUTH_QUERY = SKIP_SCANS = common.get_bool_env_var('AUTH_TEST_AUTH_QUERY', False)
-HOST = common.get_env_var('AUTH_TEST_PLANNER_HOST', common.identity, 'localhost')
 
 class AuthorizationTest(unittest.TestCase):
     @classmethod
@@ -46,7 +49,7 @@ class AuthorizationTest(unittest.TestCase):
             should be here instead of __cleanup()."""
         super(AuthorizationTest, cls).setUpClass()
         ctx = common.get_test_context()
-        with ctx.connect(HOST) as conn:
+        with common.get_planner(ctx) as conn:
             conn.delete_attribute('abac_auth_test', 'v1')
             conn.create_attribute('abac_auth_test', 'v1')
 
@@ -125,7 +128,7 @@ class AuthorizationTest(unittest.TestCase):
     # Tests that revokes to a db does not cascade to the table or columns
     def test_revoke_db_no_cascade(self):
         ctx = common.get_test_context()
-        with ctx.connect(HOST) as conn:
+        with common.get_planner(ctx) as conn:
             self.__cleanup(conn)
 
             # Grant on db, table and column
@@ -184,7 +187,9 @@ class AuthorizationTest(unittest.TestCase):
                 if not self.__is_zero(val):
                     return False
             return True
-        return v in [0, '', b''] or str(v) == '1970-01-01 00:00:00.000' or float(v) == 0
+        if v is not None and isinstance(v, (bytes, str)):
+            v = v.strip()
+        return v in [0, '', b''] or str(v) in (EPOCH0_DATE, EPOCH0_TS) or float(v) == 0
 
     # Returns the first primitive type for the schema at col/t. Returns None if
     # all leaf types are complex (e.g. array)
@@ -718,24 +723,20 @@ class AuthorizationTest(unittest.TestCase):
 
     # Tests that verify grants to complex types works. This programmatically gets
     # columns to the table/view and increasingly grants more access to it.
-    # TODO: find a library to compute powerset of columns.
     @unittest.skipIf(common.TEST_LEVEL in ["all"], "Skipping at all level")
     def test_complex_types_core(self):
         # This contains a subset that capture most cases
         ctx = common.get_test_context()
-        with ctx.connect(HOST) as conn:
-            for ds in ['struct_t', 'struct_t_view', 'struct_t_id', 'struct_nested_s1_f1',
-                       'struct_nested_s1_s2', 'struct_t_s1', 'struct_nested_s1',
-                       'strarray_t', 'strarray_t_view',
-                       'array_struct_array', 'view_over_multiple_structs']:
+        with common.get_planner(ctx) as conn:
+            for ds in ['struct_t', 'strarray_t_view']:
                 self.__test_dataset(ctx, conn, 'rs_complex', ds)
 
-    @unittest.skipIf(common.TEST_LEVEL in ["smoke", "all"], "Skipping at all level")
+    @unittest.skipIf(common.TEST_LEVEL in SKIP_LEVELS, "Skipping at unit/all level")
     def test_complex_types_extended(self):
         # This adds some additional tables and run at a higher test level to control
         # test times.
         ctx = common.get_test_context()
-        with ctx.connect(HOST) as conn:
+        with common.get_planner(ctx) as conn:
             for ds in ['struct_t2', 'struct_t3', 'struct_array_struct', 'struct_view',
                        'struct_t_restricted_view', 'struct_nested', 'struct_t_view2',
                        'struct_nested_view', 'array_struct_array', 'array_struct_t',
@@ -743,20 +744,19 @@ class AuthorizationTest(unittest.TestCase):
                        'user_phone_numbers', 'user_phone_numbers_map']:
                 self.__test_dataset(ctx, conn, 'rs_complex', ds)
 
-    @unittest.skipIf(common.TEST_LEVEL in ["all"], "Skipping at all level")
+    @unittest.skipIf(common.TEST_LEVEL in SKIP_LEVELS, "Skipping at unit/all level")
     def test_market_data(self):
         ctx = common.get_test_context()
-        with ctx.connect(HOST) as conn:
+        with common.get_planner(ctx) as conn:
             self.__test_dataset(ctx, conn, 'rs_complex', 'market_v20_single')
-            self.__test_dataset(ctx, conn, 'rs_complex', 'market_v30_single')
 
-    @unittest.skipIf(common.TEST_LEVEL in ["smoke", "all"], "Skipping at unit test level")
+    @unittest.skipIf(common.TEST_LEVEL in SKIP_LEVELS, "Skipping at unit/all level")
     def test_complex_types_slow(self):
         # These tests are N^2 with the number of columns so don't run these by default
         slow = ['test_bucketing_tbl', 'zd1216', 'zd1216_with_subscriptionlimit']
 
         ctx = common.get_test_context()
-        with ctx.connect(HOST) as conn:
+        with common.get_planner(ctx) as conn:
             for ds in slow:
                 self.__test_dataset(ctx, conn, 'rs_complex', ds, True)
             for ds in ['ledger_balance', 'subscription', 'subscription_currency',
@@ -765,7 +765,8 @@ class AuthorizationTest(unittest.TestCase):
 
     @staticmethod
     def check_dataset_metadata(database, dataset, prop, value):
-        with common.get_test_context().connect(HOST) as conn:
+        ctx = common.get_test_context()
+        with common.get_planner(ctx) as conn:
             ds_info = conn.list_datasets(db=database, name=dataset)
             assert(len(ds_info) == 1)
             ds_md = ds_info[0].metadata
@@ -783,7 +784,10 @@ class AuthorizationTest(unittest.TestCase):
 
             # These datasets/dbs take a long time to run as the dataset is big
             if ds in ['buckets_test.nytaxi_sample_bucketed',
+                      'nytaxi.orc_data',
                       'nytaxi.parquet_data',
+                      'nytaxi.parquet_mega',
+                      'nytaxi.parquet_data_symlink',
                       'nytaxi.unionallview',
                       'partition_test.part_flat_200_data_100',
                       'rs.alltypes_large_s3',
@@ -792,7 +796,11 @@ class AuthorizationTest(unittest.TestCase):
                       'rs.alltypes_large_s3_lzo']:
                 run_scan = False
 
-            if db in ['tpcds_csv_10', 'tpcds_csv_100', 'tpch_sf1', 'tpch_sf5',
+            if db in ['tpcds1000_partitioned', 'tpcds1000_unpartitioned',
+                      'tpcds100_partitioned', 'tpcds100_unpartitioned',
+                      'tpcds10_partitioned', 'tpcds10_unpartitioned',
+                      'tpcds1_partitioned', 'tpcds1_unpartitioned',
+                      'tpcds_csv_10', 'tpcds_csv_100', 'tpch_sf1', 'tpch_sf5',
                       'zd1179large']:
                 run_scan = False
 
@@ -833,21 +841,16 @@ class AuthorizationTest(unittest.TestCase):
                       'partition_test.timestamp_part_test2']:
                 continue
 
-            # For json, array of arrays is not supported
-            if ds in ['rs_json_format.json_array_arrays']:
-                run_scan = False
-
-            # BUG[DAS-4797]: fails when running count(*) queries
-            # This fails to scan.
-            if ds in ['jdbc_demo_test.transactions',
-                      'jdbc_demo_test.transactions2']:
-                run_scan = False
-
             # BUG[DAS-4799]: This crashes the worker
             # This fails to scan.
             if ds in ['parquet_testing.dict_page_offset_zero']:
                 run_scan = False
 
+            # BUG[DAS-5908]: This crashes the worker
+            # This fails to scan.
+            if ds in ['rs_complex.unnest1', 'rs_complex.unnest2']:
+                run_scan = False
+
             # BUG[DAS-4801]: Worker fails this query with nested array items
             # This fails to scan.
             if ds in ['parquet_testing.nested_lists_snappy',
@@ -901,36 +904,40 @@ class AuthorizationTest(unittest.TestCase):
     @unittest.skip("Example for use during development to test a specific dataset.")
     def test_single_dataset(self):
         ctx = common.get_test_context()
-        with ctx.connect(HOST) as conn:
+        with common.get_planner(ctx) as conn:
             self.__test_dataset(
-                ctx, conn, 'rs_complex', 'struct_t_view', minimal=False, test_scan=True)
+                ctx, conn, 'rs_complex', 'struct_view', minimal=False, test_scan=True)
 
-    @unittest.skipIf(common.TEST_LEVEL in ["smoke", "all"], "Skipping at unit test level")
+    @unittest.skipIf(common.TEST_LEVEL in SKIP_LEVELS, "Skipping at unit/all level")
     def test_rs_db(self):
         ctx = common.get_test_context()
-        with ctx.connect(HOST) as conn:
-            self.__test_db(ctx, conn, 'rs', True)
+        with common.get_planner(ctx) as conn:
+            for ds in ['alltypes', 'alltypes_null', 'ccn',
+                       'large_decimals', 'nation_projection',
+                       's3_nation', 'users']:
+                self.__test_dataset(ctx, conn, 'rs', ds)
 
-    @unittest.skipIf(common.TEST_LEVEL in ["smoke", "all"], "Skipping at unit test level")
+    @unittest.skipIf(common.TEST_LEVEL in SKIP_LEVELS, "Skipping at unit/all level")
     def test_auth_db(self):
         ctx = common.get_test_context()
-        with ctx.connect(HOST) as conn:
+        with common.get_planner(ctx) as conn:
             self.__test_db(ctx, conn, 'authdb', True)
 
     @unittest.skipIf(common.TEST_LEVEL != "all", "Skipping at unit test level")
     def test_all(self):
         # Runs test again all tables in all dbs, except the black listed ones
         ctx = common.get_test_context()
-        with ctx.connect(HOST) as conn:
+        with common.get_planner(ctx) as conn:
             dbs = conn.list_databases()
             for db in dbs:
                 # Dbs that either have public access or non queryable tables
                 # BUG DAS-5544 filter pushdown on jdbc tables
+                # Delta tables are getting errors from the manifest file when scanning
                 if db in ['bad_metadata_db', 'cerebro_sample', 'default', 'demo',
                           'okera_sample', 'okera_system', 'all_table_types',
                           'jdbc_test_mysql', 'jdbc_test_psql', 'jdbc_test_redshift',
                           'jdbc_test_snowflake', 'jdbc_test_sqlserver',
-                          'jdbc_test_oracle']:
+                          'jdbc_test_oracle', 'delta_db']:
                     continue
                 if db.startswith('_'):
                     continue
