@@ -12,20 +12,25 @@
 # pylint: disable=no-self-use
 # pylint: disable=line-too-long
 # pylint: disable=too-many-locals
+# pylint: disable=pointless-string-statement
 import unittest
 
 from decimal import Decimal
 import numpy
+import os
 from tzlocal import get_localzone
+import pytest
 
 from okera import version, _thrift_api
 from okera.tests import pycerebro_test_common as common
-from okera._thrift_api import TAccessPermissionLevel
+from okera._thrift_api import TAccessPermissionLevel, TRecordServiceException
 
 # The timestamp values are TZ specific, switch based on this.
 # TODO: this is an awful way to do this.
 TIME_ZONE = get_localzone().zone
 
+DEFAULT_PRESTO_PORT = os.environ['ODAS_TEST_PORT_PRESTO_COORD_HTTPS']
+
 class BasicTest(unittest.TestCase):
     @staticmethod
     def __table_exists(name, tbls):
@@ -587,6 +592,9 @@ class BasicTest(unittest.TestCase):
         DB2 = 'test_access_db2'
         DB3 = 'test_access_db3'
         DB4 = 'test_access_db4'
+        DB5 = 'test_access_db5'
+        DB6 = 'test_access_db6'
+        DB7 = 'test_access_db7'
         TBL1 = 'tbl1'
         TBL2 = 'tbl2'
         ROLE = 'test_access_level_role'
@@ -599,6 +607,9 @@ class BasicTest(unittest.TestCase):
                 for tbl in [TBL1, TBL2]:
                     conn.execute_ddl('CREATE TABLE %s.%s (col1 int, col2 int)' % (
                         db, tbl))
+            for db in [DB5, DB6, DB7]:
+                conn.execute_ddl('DROP DATABASE IF EXISTS %s CASCADE' % db)
+                conn.execute_ddl('CREATE DATABASE %s' % db)
 
             conn.execute_ddl('CREATE ATTRIBUTE IF NOT EXISTS %s.%s' % (ATTR[0], ATTR[1]))
             conn.execute_ddl('DROP ROLE IF EXISTS %s' % ROLE)
@@ -609,6 +620,9 @@ class BasicTest(unittest.TestCase):
             conn.execute_ddl('''GRANT SELECT ON TABLE %s.%s
                                 HAVING ATTRIBUTE %s.%s
                                 TO ROLE %s''' % (DB3, TBL1, ATTR[0], ATTR[1], ROLE))
+            conn.execute_ddl('GRANT CREATE ON DATABASE %s TO ROLE %s' % (DB5, ROLE))
+            conn.execute_ddl('GRANT CREATE_AS_OWNER ON DATABASE %s TO ROLE %s' % (DB6, ROLE))
+            conn.execute_ddl('GRANT ALL ON DATABASE %s TO ROLE %s' % (DB7, ROLE))
             conn.execute_ddl('ALTER TABLE %s.%s ADD ATTRIBUTE %s.%s' % (
                 DB3, TBL1, ATTR[0], ATTR[1]))
             conn.execute_ddl('GRANT ROLE %s TO GROUP %s' % (ROLE, USER))
@@ -661,6 +675,7 @@ class BasicTest(unittest.TestCase):
             assert DB2 in dbs
             assert DB3 in dbs
             assert DB4 not in dbs
+            assert DB7 in dbs
 
             # This will list all databases with view audit privilege
             dbs = conn.list_databases(privilege=TAccessPermissionLevel.VIEW_AUDIT)
@@ -668,6 +683,7 @@ class BasicTest(unittest.TestCase):
             assert DB2 in dbs
             assert DB3 not in dbs
             assert DB4 not in dbs
+            assert DB7 in dbs
 
             # This will list all databases with SELECT (for which all but one of
             # our DBs won't be in)
@@ -676,6 +692,57 @@ class BasicTest(unittest.TestCase):
             assert DB2 not in dbs
             assert DB3 in dbs
             assert DB4 not in dbs
+            assert DB7 in dbs
+
+            # This will list all databases with SELECT and VIEW_AUDIT
+            dbs = conn.list_databases(
+                privilege=[TAccessPermissionLevel.SELECT, TAccessPermissionLevel.VIEW_AUDIT])
+            assert DB1 in dbs
+            assert DB2 in dbs
+            assert DB3 in dbs
+            assert DB4 not in dbs
+            assert DB5 not in dbs
+            assert DB7 in dbs
+
+            # This will list all databases with CREATE
+            dbs = conn.list_databases(privilege=TAccessPermissionLevel.CREATE)
+            assert DB5 in dbs
+            assert DB6 not in dbs
+            assert DB7 in dbs
+
+            # This will list all databases with CREATE_AS_OWNER
+            dbs = conn.list_databases(privilege=TAccessPermissionLevel.CREATE_AS_OWNER)
+            assert DB5 not in dbs
+            assert DB6 in dbs
+            assert DB7 in dbs
+
+            # This will list all databases with ALL
+            dbs = conn.list_databases(privilege=TAccessPermissionLevel.ALL)
+            assert DB5 not in dbs
+            assert DB6 not in dbs
+            assert DB7 in dbs
+
+            # This will list all databases with CREATE and CREATE_AS_OWNER
+            dbs = conn.list_databases(
+                privilege=[TAccessPermissionLevel.ALL, TAccessPermissionLevel.CREATE, TAccessPermissionLevel.CREATE_AS_OWNER])
+            assert DB5 in dbs
+            assert DB6 in dbs
+            assert DB7 in dbs
+
+    def test_quarantine_dbs(self):
+        ctx = common.get_test_context()
+        ctx.enable_token_auth(token_str='root')
+        with common.get_planner(ctx, presto_port=DEFAULT_PRESTO_PORT) as conn:
+            conn.execute_ddl("drop crawler if exists okera_quarantine")
+            conn.execute_ddl("drop database if exists okera_quarantine")
+            conn.execute_ddl("create database okera_quarantine")
+            conn.execute_ddl("create crawler okera_quarantine source 's3://cerebrodata-test/empty/'")
+            result = conn.list_databases()
+            assert "okera_quarantine" not in result
+            assert "_okera_crawler_okera_quarantine" not in result
+            result_presto = conn.scan_as_pandas("show schemas", dialect='presto')
+            assert "okera_quarantine" not in result_presto
+            assert "_okera_crawler_okera_quarantine" not in result_presto
 
     def test_get_datasets_with_low_privs(self):
         ctx = common.get_test_context()
@@ -735,5 +802,80 @@ class BasicTest(unittest.TestCase):
         dbs = conn.list_databases()
         assert 'okera_sandbox' not in dbs
 
+    def test_create_unparseable_view(self):
+        ctx = common.get_test_context()
+        with common.get_planner(ctx) as planner:
+            test_db = "invalid_view_db"
+            ## Create a view with unparseable query.
+            test_view = "unparseable_view"
+
+            planner.execute_ddl("DROP DATABASE IF EXISTS %s CASCADE" % test_db)
+            planner.execute_ddl("CREATE DATABASE %s " % test_db)
+
+            # Verify that we cannot CREATE internal view with unparseable statement.
+            try:
+                planner.execute_ddl(
+                    "CREATE VIEW %s.%s as select 1 as `#foo`" % (test_db, test_view))
+                pytest.fail("Expected to fail parsing unparseable view %s.%s"
+                    % (test_db, test_view))
+            except TRecordServiceException as ex:
+                print(ex)
+                self.assertTrue("AnalysisException" in ex.detail)
+                self.assertTrue("TableLoadingException" in ex.detail)
+                self.assertTrue(
+                    "Failed to parse view-definition statement of view: %s.%s due to"
+                    % (test_db, test_view) in ex.detail)
+                self.assertTrue("#foo" in ex.detail)
+
+            # Verify that we cannot ALTER internal view with unparseable statement.
+            planner.execute_ddl(
+                "CREATE VIEW %s.%s as select 1 as `foo`" % (test_db, test_view))
+            try:
+                planner.execute_ddl(
+                    "ALTER VIEW %s.%s as select 1 as `#foo`" % (test_db, test_view))
+                pytest.fail("Expected to fail parsing unparseable view %s.%s"
+                    % (test_db, test_view))
+            except TRecordServiceException as ex:
+                print(ex)
+                self.assertTrue("AnalysisException" in ex.detail)
+                self.assertTrue("TableLoadingException" in ex.detail)
+                self.assertTrue(
+                    "Failed to parse view-definition statement of view: %s.%s due to"
+                    % (test_db, test_view) in ex.detail)
+                self.assertTrue("#foo" in ex.detail)
+
+            # Verify that we DO NOT ALLOW CREATE external view with unparseable statement.
+            try:
+              planner.execute_ddl("DROP VIEW %s.%s " % (test_db, test_view))
+              planner.execute_ddl(
+                  "CREATE EXTERNAL VIEW %s.%s as select 1 as `#foo`"
+                  % (test_db, test_view))
+              pytest.fail("Expected to fail parsing unparseable view %s.%s"
+                    % (test_db, test_view))
+            except TRecordServiceException as ex:
+                print(ex)
+                self.assertTrue("AnalysisException" in ex.detail)
+                self.assertTrue("TableLoadingException" in ex.detail)
+                self.assertTrue(
+                    "Failed to parse view-definition statement of view: %s.%s due to"
+                    % (test_db, test_view) in ex.detail)
+                self.assertTrue("#foo" in ex.detail)
+
+            # Verify that we ALLOW CREATE external view with unparseable statement and
+            # skip_analysis keyword.
+            planner.execute_ddl(
+                "CREATE EXTERNAL VIEW %s.%s (id int) skip_analysis " \
+                "as select 1 as `#foo`" % (test_db, test_view))
+
+            # External view with unparseable query should load for describe statements.
+            res = planner.execute_ddl("DESCRIBE %s.%s " % (test_db, test_view))
+            self.assertTrue(len(res) == 1)
+            res = planner.execute_ddl("DESCRIBE FORMATTED %s.%s " % (test_db, test_view))
+            self.assertTrue(test_db in res)
+            self.assertTrue("VIRTUAL_VIEW" in res)
+            self.assertTrue("#foo" in res)
+
+            planner.execute_ddl("DROP DATABASE IF EXISTS %s CASCADE" % test_db)
+
 if __name__ == "__main__":
     unittest.main()
